﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Windows.Data;
using System.Windows.Media;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

namespace ScheduleViewTest
{
	public class MyCollectionViewFactory : ObservableCollection<Appointment>, ICollectionViewFactory
	{
		public object Source { get; set; }

		public MyCollectionViewFactory(IEnumerable<object> source)
		{
			this.Source = source;
		}

		public ICollectionView CreateView()
		{
			var collectionViewSource = new CollectionViewSource { Source = this.Source };
			return collectionViewSource.View;
		}
	}

	public class Model : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;
        private void RaisePropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        private MyCollectionViewFactory m_Orders_BusOC;
        public MyCollectionViewFactory Orders_BusOC
        {
            get
            {
                if (m_Orders_BusOC == null)
                    m_Orders_BusOC = new MyCollectionViewFactory(null);

                return m_Orders_BusOC;
            }
            set
            {
                m_Orders_BusOC = value;
                RaisePropertyChanged("Orders_BusOC");
            }
        }

        private ResourceTypeCollection m_ResourceTypes1;
        public ResourceTypeCollection ResourceTypes1
        {
            get { return m_ResourceTypes1; }
            set
            {
                m_ResourceTypes1 = value;
                RaisePropertyChanged("ResourceTypes1");
            }
        }
        private ResourceTypeCollection m_ResourceTypes2;
        public ResourceTypeCollection ResourceTypes2
        {
            get { return m_ResourceTypes2; }
            set
            {
                m_ResourceTypes2 = value;
                RaisePropertyChanged("ResourceTypes2");
            }
        }

        private DateTime m_CurrentDate;
        public DateTime CurrentDate
        {
            get { return m_CurrentDate; }
            set
            {
                m_CurrentDate = value;
                RaisePropertyChanged("CurrentDate");
            }
        }
        
        public Model()
        {
            var day = (int)DateTime.Today.DayOfWeek;
            var czechDay = (day - 1) % 7;
            CurrentDate = DateTime.Today.AddDays(-czechDay);
            
            Orders_BusOC = new MyCollectionViewFactory(GetOrdersByBus());

            var busResources = GetBusResources();

            // here is the core of the issue:

            var r1 = busResources.Where(r => r.ResourceId <= 2);
            ResourceTypes1 = GetBusResourceTypeCollection(r1);

            var r2 = busResources.Where(r => r.ResourceId > 2);
            ResourceTypes2 = GetBusResourceTypeCollection(r2);

            RaisePropertyChanged("AppoitmentsFilter");
        }

        public IEnumerable<OrderStrip> GetOrdersByBus()
        {
            return
               (from i in Enumerable.Range(1, 8) // rows
                from j in Enumerable.Range(0, 3) // cols
                let enter = CurrentDate.AddDays(j * 2)
                let resourceId = i
                select new { i, resourceId, enter })
               .Select(t =>
                {
                    var app = new OrderStrip
                    {
                        Row = t.i,
                        Start = t.enter,
                        End = t.enter.AddDays(1),
                        Subject = $"Res id: {t.resourceId}",
                    };
                    var r = new OrderResource(t.resourceId.ToString(), "Bus") { DisplayName = $"Res id: {t.resourceId}" };
                    app.Resources.Add(r);
                    return app;
                })
                .ToArray();
        }

        public IEnumerable<OrderResource> GetBusResources()
        {
            return
                (from i in Enumerable.Range(1, 8)
                 select new OrderResource(i.ToString(), "Bus")
                 {
                     ResourceId = i,
                     DisplayName = $"Res id: {i}",
                 });
        }

        public ResourceTypeCollection GetBusResourceTypeCollection(IEnumerable<OrderResource> resources)
        {
            var busResource = new ResourceType("Bus");
            busResource.Resources.AddRange(resources);
            return new ResourceTypeCollection() { busResource };
        }
    }

    public class OrderResource : Resource, INotifyPropertyChanged
    {
        public int? ResourceId { get; set; } // bus id or driver id
        
        private void RaisePropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        public event PropertyChangedEventHandler PropertyChanged;

        private bool m_IsSelected;
        public bool IsSelected
        {
            get { return m_IsSelected; }
            set
            {
                m_IsSelected = value;
                RaisePropertyChanged("IsSelected");
            }
        }

        public OrderResource(string name, string type) : base(name, type) { }
    }

    public class OrderStrip : Appointment
    {
        public int Row { get; set; }
    }
}
